unit UnitRegSave;
{
    Purpose:
        Save a subkey to a file or a root key to a subfolder
        using the subkey name as the target filename.
}

{///////////////}
{//}interface{//}
{///////////////}
uses Registry, SysUtils, Windows, Classes, Dialogs;

type TRegSave = class(TObject)
private
    procedure GetBackupSecurity;
public
    constructor Create;
    destructor Destroy; override;

    procedure SaveKeyToFile(root : hkey; keyname, filename : string);
    procedure SaveRootToFolder(root : hkey; foldername : string);
    
    procedure GetKeyList(root : HKEY; keys : TStringList);
end;

{////////////////////}
{//}implementation{//}
{////////////////////}


constructor TRegSave.Create;
begin
   self.GetBackupSecurity;
end;
destructor TRegSave.Destroy;
begin
    inherited destroy;
end;

procedure TRegSave.GetKeyList(root : HKEY; keys : TStringList);
var c : cardinal;
    s : string;
    maxsize : cardinal;
begin
    c := 0;
    maxsize := 255;
    setlength(s, maxsize);

    keys.Clear;
    while (Windows.RegEnumKeyEx(
        root,      {root key}
        c,         { index of key to read}
        PChar(s),  {name of subkey}
        maxsize,   {size of buffer/size of keyname returned}
        nil,       {reserved}
        nil,
        nil,
        nil
    )  <> ERROR_NO_MORE_ITEMS) do begin
        s := String(PChar(s));
        keys.Add(s);

        maxsize := 255;
        setlength(s, maxsize);
        inc(c);
    end;
end;

procedure TRegSave.SaveRootToFolder(root : hkey; foldername : string);
var i : integer;
    keys : TStringList;
begin
    keys := TStringList.Create;
    self.GetKeyList(root, keys);
    for i := 0 to (keys.count - 1) do begin
        self.SaveKeyToFile(root, keys[i], IncludeTrailingPathDelimiter(foldername) + keys[i]);
    end;
end;


procedure TRegSave.GetBackupSecurity;
var hToken: cardinal;
    luid : Int64;
    tp : _TOKEN_PRIVILEGES;
    rl : cardinal;
begin
    //
    // enable backup privilege
    //
    if( not Windows.OpenProcessToken(Windows.GetCurrentProcess(),
                        TOKEN_ADJUST_PRIVILEGES,
                        hToken )) then begin
        Raise  Exception.Create('GetBackupSecurity1: ' + SysErrorMessage(GetLastError));
        EXIT;
    end;
    if (not Windows.LookupPrivilegeValue(nil, PChar('SeBackupPrivilege'), luid)) then begin
        raise Exception.Create(SysErrorMessage(GetLastError));
        EXIT;
    end;

    tp.PrivilegeCount           := 1;
    tp.Privileges[0].Luid       := luid;
    tp.Privileges[0].Attributes := SE_PRIVILEGE_ENABLED;
    if (not Windows.AdjustTokenPrivileges(hToken, FALSE, tp, sizeof(TOKEN_PRIVILEGES),
                                nil, rl)) then begin
        raise Exception.Create('GetBackupSecurity2: ' + SysErrorMessage(GetLastError));
        EXIT;
    end;
end;


procedure TRegSave.SaveKeyToFile(root : hkey; keyname, filename : string);
var K : HKEY;
    Disposition : DWORD;
begin
    if (Windows.RegCreateKeyEx(
        root,                      {root key}
        PChar(keyname),            {subkey}
        0,                         {Must be zero}
        nil,                       {key already exists, ignored}
        REG_OPTION_BACKUP_RESTORE, {required to access SECURITY}
        KEY_QUERY_VALUE,           {read only}
        nil,                       {use default security descripter}
        K,                         {Key handle returned}
        @Disposition)              {returns a Created or Opened value}
        <> ERROR_SUCCESS) then begin
        raise Exception.Create('SaveKeyToFile1: ' + SysErrorMessage(GetLastError));
        EXIT;
    end;

    if (Windows.RegSaveKey(K,   {key handle to save}
        PChar(filename),   {file to save the key}
        nil                     {use default security descriptor}
        ) <>
        ERROR_SUCCESS) then begin
        raise Exception.Create('SaveKeyToFile1: ' + SysErrorMessage(GetLastError));
        EXIT;
    end;
    Windows.RegCloseKey(K);
end;



end.
